"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const param_case_1 = require("param-case");
const pascal_case_1 = require("pascal-case");
const path_1 = __importDefault(require("path"));
const vue_docgen_api_1 = require("vue-docgen-api");
class default_1 {
    constructor() {
        this.supportedFileExtensions = [".vue"];
        this.docgenOptions = {};
    }
    supports(context) {
        const fileExtension = path_1.default.extname(context.path);
        return this.supportedFileExtensions.includes(fileExtension);
    }
    init() {
        const webpackConfig = this.attemptToRetrieveVueCliProjectWebpackConfig();
        if (webpackConfig && webpackConfig.resolve && webpackConfig.resolve.alias) {
            this.docgenOptions = {
                alias: webpackConfig.resolve.alias
            };
        }
        return Promise.resolve();
    }
    process(context) {
        return __awaiter(this, void 0, void 0, function* () {
            const lang = "html" /* HTML */;
            const componentInfo = yield vue_docgen_api_1.parse(context.path, this.docgenOptions);
            const { description, displayName } = componentInfo;
            const propLines = this.getPropLines(componentInfo);
            const eventLines = this.getEventLines(componentInfo);
            const slotLines = this.getSlotLines(componentInfo);
            const snippet = this.generateSnippet(displayName, propLines, eventLines, slotLines);
            return { description, snippet, lang };
        });
    }
    /**
     * Note: If we eventually want to support aliases from non-Vue CLI projects,
     * we'll likely want to add a test for this and other methods to retrieve those aliases
     * For now, it's not worth the effort to setup & maintain an entire dummy Vue CLI project,
     * with the relevant vue.config.js
     */
    attemptToRetrieveVueCliProjectWebpackConfig() {
        let webpackConfig;
        try {
            // Documentation on how @vue/cli resolves the Webpack config for this file: 
            // https://cli.vuejs.org/guide/webpack.html#using-resolved-config-as-a-file
            // eslint-disable-next-line @typescript-eslint/no-var-requires
            const vueCliWebpackConfig = require('@vue/cli-service/webpack.config.js');
            webpackConfig = vueCliWebpackConfig;
        }
        catch (_a) {
            console.log('Did not resolve webpack config for Vue CLI project; proceeding without aliases for parsing components');
        }
        return webpackConfig;
    }
    getPropLines(componentInfo) {
        return componentInfo.props && componentInfo.props.map(prop => `${param_case_1.paramCase(prop.name)}="${prop.type && prop.type.name || 'unknown'}"`) || [];
    }
    getSlotLines(componentInfo) {
        return componentInfo.slots && componentInfo.slots.map(slot => {
            if (slot.name === 'default')
                return `<!-- content -->`;
            if (slot.scoped) {
                let slotBinding = 'scopedBinding';
                if (slot.bindings && slot.bindings[0] && slot.bindings[0].name)
                    slotBinding = slot.bindings[0].name;
                return `<template #${slot.name}="${slotBinding}"></template>`;
            }
            return `<template #${slot.name}></template>`;
        }) || [];
    }
    getEventLines(componentInfo) {
        return componentInfo.events && componentInfo.events.map(event => {
            return `@${event.name}="handle${pascal_case_1.pascalCase(event.name)}"`;
        }) || [];
    }
    generateSnippet(displayName, propLines, eventLines, slotLines) {
        let snippet = '';
        const combinedPropEventLines = [...propLines, ...eventLines];
        if (!combinedPropEventLines.length && !slotLines.length) {
            snippet = `<${displayName} />`;
        }
        else if (combinedPropEventLines.length && !slotLines.length) {
            snippet =
                `<${displayName}
  ${combinedPropEventLines.join('\n  ')}
/>`;
        }
        else if (!combinedPropEventLines.length && slotLines.length) {
            snippet =
                `<${displayName}>
  ${slotLines.join('\n  ')}
</${displayName}>`;
        }
        else {
            snippet =
                `<${displayName}
  ${combinedPropEventLines.join('\n  ')}
>
  ${slotLines.join('\n  ')}
</${displayName}>`;
        }
        return snippet;
    }
}
exports.default = default_1;
//# sourceMappingURL=index.js.map